==> models_weebi-1.0.22.tar.gz: lib/src/extensions/tickets_stock_cl_report.dart <==
import 'package:models_weebi/src/reports/report_stock_article_calibre.dart';
import 'package:models_weebi/src/models/article_calibre.dart';
import 'package:models_weebi/src/models/ticket_weebi.dart';

import 'package:models_weebi/src/models/utils/date_range.dart';

extension IsInRange on DateTime {
  bool isInRange(DateTime startDateRange, DateTime endDateRange) =>
      (isAfter(startDateRange) || isAtSameMomentAs(startDateRange)) &&
      (isBefore(endDateRange) || isAtSameMomentAs(endDateRange));
}

extension StockTickets on Iterable<TicketWeebi> {
  List<ReportStockProduct> herderTicketsStock(
      String herderId,
      DateRange dateRange,
      List<ArticleCalibre> products,
      List<ReportStockProduct> emptyReportStockProducts) {
    final _products = products.where((p) => p.id != 0);

    forEach((t) {
      if (t.contactId == herderId && t.status) {
        if (t.date.isInRange(dateRange.start, dateRange.end)) {
          for (final p in _products) {
            if (t.items.any((i) => i.article.productId == p.id)) {
              emptyReportStockProducts
                      .firstWhere((rPS) => rPS.id == p.id)
                      .varQtTickets +=
                  t.items.where((i) => i.article.productId == p.id).fold(0.0,
                      (p, i) {
                // herder logic is reversed from the shop one
                // one day herder might buy and sell same product
                // which could then be input or ouput
                if (t.ticketType.isShopOutput) {
                  // print('p + i.quantity ${i.quantity}');
                  return p + i.quantity;
                } else {
                  // print('p - i.quantity ${i.quantity}');
                  return p - i.quantity;
                }
                // * ToRuminate : also include emptyReportStockProducts.articles
              });
            }
          }
        }
        // if (t.date.isInRange(previousMonthStart, previousMonthEnd)) {
        //   for (final p in _products) {
        //     if (t.items.any((i) => i.article.productId == p.id)) {
        //       emptyReportStockProducts
        //               .firstWhere((rPS) => rPS.id == p.id)
        //               .initialQtTickets +=
        //           t.items.where((i) => i.article.productId == p.id).fold(0.0,
        //               (p, i) {
        //         // herder logic is reversed from the shop one
        //         // one day herder might buy and sell same product
        //         // which could then be input or ouput
        //         if (t.ticketType.isShopOutput) {
        //           // print('p + i.quantity ${i.quantity}');
        //           return p + i.quantity;
        //         } else {
        //           // print('p - i.quantity ${i.quantity}');
        //           return p - i.quantity;
        //         }
        //         // * ToRuminate : also include emptyReportStockProducts.articles
        //       });
        //     }
        //   }
        // }
      }
    });
    return emptyReportStockProducts;
  }

  List<ReportStockProduct> shopTicketsStock(
      String shopUuid,
      DateRange dateRange,
      List<ArticleCalibre> products,
      List<ReportStockProduct> emptyReportStockProducts) {
    final _products = products.where((p) => p.id != 0);
    // no way to get initialQtTickets other than computing pastMonth finalQt

    forEach((t) {
      if (t.shopUuid == shopUuid && t.status) {
        // filtering this month
        if (t.date.isInRange(dateRange.start, dateRange.end)) {
          for (final p in _products) {
            if (t.items.any((i) => i.article.productId == p.id)) {
              var rp =
                  emptyReportStockProducts.firstWhere((rP) => rP.id == p.id);
              rp.varQtTickets += t.items
                  .where((i) => i.article.productId == p.id)
                  .fold(0.0, (p, item) {
                // * one product can be input or ouput
                //print('item.quantity ${item.quantity} t.ticketType.isShopOutput ${t.ticketType.isShopOutput}');
                if (t.ticketType.isShopOutput) {
                  return p - item.quantity;
                } else {
                  return p + item.quantity;
                }
              });
            }
          }
        }
        // } else if (t.date.isInRange(previousMonthStart, previousMonthEnd)) {
        //   for (final p in _products) {
        //     if (t.items.any((i) => i.article.productId == p.id)) {
        //       var rp = emptyReportStockProducts.firstWhere((rP) => rP.id == p.id);
        //       rp.initialQtTickets += t.items
        //           .where((i) => i.article.productId == p.id)
        //           .fold(0.0, (p, item) {
        //         // * one product can be input or ouput
        //         //print('item.quantity ${item.quantity} t.ticketType.isShopOutput ${t.ticketType.isShopOutput}');
        //         if (t.ticketType.isShopOutput) {
        //           return p - item.quantity;
        //         } else {
        //           return p + item.quantity;
        //         }
        //       });
        //     }
        //   }
      }
    });
    return emptyReportStockProducts;
  }
}

==> models_weebi-1.1.1.tar.gz: lib/src/extensions/tickets_stock_cl_report.dart <==
import 'package:models_weebi/src/reports/report_stock_article_calibre.dart';
import 'package:models_weebi/src/models/article_calibre.dart';
import 'package:models_weebi/src/models/ticket_weebi.dart';

import 'package:models_weebi/src/models/utils/date_range.dart';

extension IsInRange on DateTime {
  bool isInRange(DateTime startDateRange, DateTime endDateRange) =>
      (isAfter(startDateRange) || isAtSameMomentAs(startDateRange)) &&
      (isBefore(endDateRange) || isAtSameMomentAs(endDateRange));
}

extension StockTickets on Iterable<TicketWeebi> {
  List<ReportStockProduct> herderTicketsStock(
      int herderId,
      DateRange dateRange,
      List<ArticleCalibre> products,
      List<ReportStockProduct> emptyReportStockProducts) {
    final products0 = products.where((p) => p.id != 0);

    forEach((t) {
      if (t.contactId == herderId && t.status) {
        if (t.date.isInRange(dateRange.start, dateRange.end)) {
          for (final p in products0) {
            if (t.items.any((i) => i.article.productId == p.id)) {
              emptyReportStockProducts
                      .firstWhere((rPS) => rPS.id == p.id)
                      .varQtTickets +=
                  t.items.where((i) => i.article.productId == p.id).fold(0.0,
                      (p, i) {
                // herder logic is reversed from the shop one
                // one day herder might buy and sell same product
                // which could then be input or ouput
                if (t.ticketType.isShopOutput) {
                  // print('p + i.quantity ${i.quantity}');
                  return p + i.quantity;
                } else {
                  // print('p - i.quantity ${i.quantity}');
                  return p - i.quantity;
                }
                // * ToRuminate : also include emptyReportStockProducts.articles
              });
            }
          }
        }
        // if (t.date.isInRange(previousMonthStart, previousMonthEnd)) {
        //   for (final p in _products) {
        //     if (t.items.any((i) => i.article.productId == p.id)) {
        //       emptyReportStockProducts
        //               .firstWhere((rPS) => rPS.id == p.id)
        //               .initialQtTickets +=
        //           t.items.where((i) => i.article.productId == p.id).fold(0.0,
        //               (p, i) {
        //         // herder logic is reversed from the shop one
        //         // one day herder might buy and sell same product
        //         // which could then be input or ouput
        //         if (t.ticketType.isShopOutput) {
        //           // print('p + i.quantity ${i.quantity}');
        //           return p + i.quantity;
        //         } else {
        //           // print('p - i.quantity ${i.quantity}');
        //           return p - i.quantity;
        //         }
        //         // * ToRuminate : also include emptyReportStockProducts.articles
        //       });
        //     }
        //   }
        // }
      }
    });
    return emptyReportStockProducts;
  }

  List<ReportStockProduct> shopTicketsStock(
      int shopId,
      DateRange dateRange,
      List<ArticleCalibre> products,
      List<ReportStockProduct> emptyReportStockProducts) {
    final products0 = products.where((p) => p.id != 0);
    // no way to get initialQtTickets other than computing pastMonth finalQt

    forEach((t) {
      // * for dashbord display of multiple shop also filter with below
      // if(t.shopId == shopId )
      if (t.status) {
        // filtering this month
        if (t.date.isInRange(dateRange.start, dateRange.end)) {
          for (final p in products0) {
            if (t.items.any((i) => i.article.productId == p.id)) {
              var rp =
                  emptyReportStockProducts.firstWhere((rP) => rP.id == p.id);
              rp.varQtTickets += t.items
                  .where((i) => i.article.productId == p.id)
                  .fold(0.0, (p, item) {
                // * one product can be input or ouput
                //print('item.quantity ${item.quantity} t.ticketType.isShopOutput ${t.ticketType.isShopOutput}');
                if (t.ticketType.isShopOutput) {
                  return p - item.quantity;
                } else {
                  return p + item.quantity;
                }
              });
            }
          }
        }
        // } else if (t.date.isInRange(previousMonthStart, previousMonthEnd)) {
        //   for (final p in _products) {
        //     if (t.items.any((i) => i.article.productId == p.id)) {
        //       var rp = emptyReportStockProducts.firstWhere((rP) => rP.id == p.id);
        //       rp.initialQtTickets += t.items
        //           .where((i) => i.article.productId == p.id)
        //           .fold(0.0, (p, item) {
        //         // * one product can be input or ouput
        //         //print('item.quantity ${item.quantity} t.ticketType.isShopOutput ${t.ticketType.isShopOutput}');
        //         if (t.ticketType.isShopOutput) {
        //           return p - item.quantity;
        //         } else {
        //           return p + item.quantity;
        //         }
        //       });
        //     }
        //   }
      }
    });
    return emptyReportStockProducts;
  }
}
